@@ -1,16 +1,23 @@
 from __future__ import annotations
 
-import subprocess
+import os
+import re
+
+import traceback
 from typing import Any, Dict, List, Optional, Union
 
 import e3.collection.dag
 from e3.fs import rm, sync_tree
-from e3.os.process import get_rlimit, quote_arg
+from e3.os.process import DEVNULL, PIPE, Run, STDOUT, quote_arg
+from e3.testsuite import CleanupMode
 from e3.testsuite.utils import DummyColors
-from e3.testsuite.control import (TestControl, TestControlCreator,
-                                  YAMLTestControlCreator)
+from e3.testsuite.control import (
+    TestControl,
+    TestControlCreator,
+    YAMLTestControlCreator,
+)
 from e3.testsuite.driver import TestDriver
-from e3.testsuite.result import Log, TestStatus, truncated
+from e3.testsuite.result import Log, TestResult, TestStatus, truncated
 
 from colorama import Fore, Style
 
@@ -48,6 +55,13 @@ class TestAbortWithFailure(Exception):
     pass
 
 
+# Regular expressions to match the "timeout" error message from the rlimit
+# program.
+TIMEOUT_OUTPUT_PATTERN = r"rlimit: Real time limit ([^\n]+) exceeded\n"
+TIMEOUT_OUTPUT_STR_RE = re.compile(TIMEOUT_OUTPUT_PATTERN)
+TIMEOUT_OUTPUT_BYTES_RE = re.compile(TIMEOUT_OUTPUT_PATTERN.encode("ascii"))
+
+
 class ProcessResult:
     """Record results from a subprocess."""
 
@@ -133,15 +147,18 @@ class ClassicTestDriver(TestDriver):
         """
         return YAMLTestControlCreator({})
 
-    def shell(self,
-              args: List[str],
-              cwd: Optional[str] = None,
-              env: Optional[Dict[str, str]] = None,
-              catch_error: bool = True,
-              analyze_output: bool = True,
-              timeout: Optional[int] = None,
-              encoding: Optional[str] = None,
-              truncate_logs_threshold: Optional[int] = None) -> ProcessResult:
+    def shell(
+        self,
+        args: List[str],
+        cwd: Optional[str] = None,
+        env: Optional[Dict[str, str]] = None,
+        catch_error: bool = True,
+        analyze_output: bool = True,
+        timeout: Optional[int] = None,
+        encoding: Optional[str] = None,
+        truncate_logs_threshold: Optional[int] = None,
+        ignore_environ: bool = True,
+    ) -> ProcessResult:
         """Run a subprocess.
 
         :param args: Arguments for the subprocess to run.
@@ -163,6 +180,10 @@ class ClassicTestDriver(TestDriver):
             output in ``self.result.log``. See
             ``e3.testsuite.result.truncated``'s ``line_count`` argument. If
             left to None, use the testsuite's ``--truncate-logs`` option.
+        :param ignore_environ: Applies only when ``env`` is not None.
+            When True (the default), pass exactly environment variables
+            in ``env``. When False, pass a copy of ``os.environ`` that is
+            augmented with variables in ``env``.
         """
         # By default, run the subprocess in the test working directory
         if cwd is None:
@@ -183,34 +204,35 @@ class ClassicTestDriver(TestDriver):
                 self.Style.DIM,
                 value,
             )
+
         self.result.log += format_header(
             "Running",
             "{} (cwd={}{}{})".format(
                 " ".join(quote_arg(a) for a in args),
                 self.Style.RESET_ALL,
                 cwd,
-                self.Style.DIM
-            )
+                self.Style.DIM,
+            ),
         )
 
-        process_info = {"cmd": args,
-                        "cwd": cwd}
+        process_info = {"cmd": args, "cwd": cwd}
         self.result.processes.append(process_info)
 
-        # Python2's subprocess module does not handle timeout, so re-implement
-        # e3.os.process's rlimit-based implementation of timeouts.
-        if timeout is not None:
-            args = [get_rlimit(), str(timeout)] + args
-
-        # We cannot use e3.os.process.Run as this API forces the use of text
-        # streams, whereas testsuite sometimes need to deal with binary data
-        # (or unknown encodings, which is equivalent).
-        subp = subprocess.Popen(
-            args, cwd=cwd, env=env, stdin=subprocess.DEVNULL,
-            stdout=subprocess.PIPE, stderr=subprocess.STDOUT
+        subp = Run(
+            cmds=args,
+            cwd=cwd,
+            output=PIPE,
+            error=STDOUT,
+            input=DEVNULL,
+            timeout=timeout,
+            env=env,
+            ignore_environ=ignore_environ,
         )
+
+        # Testsuites sometimes need to deal with binary data (or unknown
+        # encodings, which is equivalent), so always use subp.raw_out.
         stdout: Union[str, bytes]
-        stdout, _ = subp.communicate()
+        stdout = subp.raw_out
         assert isinstance(stdout, bytes)
         encoding = encoding or self.default_encoding
         if encoding != "binary":
@@ -223,15 +245,20 @@ class ClassicTestDriver(TestDriver):
                     )
                 )
 
-        p = ProcessResult(subp.returncode, stdout)
+        # We run subprocesses in foreground mode, so by the time Run's
+        # constructor has returned, the subprocess is supposed to have
+        # completed, and thus we are supposed to have an exit status code.
+        assert subp.status is not None
+        p = ProcessResult(subp.status, stdout)
 
         self.result.log += format_header("Status code", p.status)
         process_info["status"] = p.status
         process_info["output"] = Log(stdout)
 
         self.result.log += format_header(
-            "Output", "\n"
-            + truncated(str(process_info["output"]), truncate_logs_threshold)
+            "Output",
+            "\n"
+            + truncated(str(process_info["output"]), truncate_logs_threshold),
         )
 
         # If requested, use its output for analysis
@@ -253,7 +280,7 @@ class ClassicTestDriver(TestDriver):
         assert not self.test_control.skip
 
         if self.test_control.xfail:
-            self.result.set_status(TestStatus.XPASS)
+            self.result.set_status(TestStatus.XPASS, self.test_control.message)
         else:
             self.result.set_status(TestStatus.PASS)
         self.push_result()
@@ -307,6 +334,33 @@ class ClassicTestDriver(TestDriver):
         """
         pass
 
+    def cleanup_working_dir(self) -> None:
+        """Remove the working directory tree."""
+        try:
+            rm(self.working_dir(), True)
+        except Exception:  # no cover
+            # TODO (U222-013) For mysterious reasons, on Windows hosts,
+            # sometimes executable files are still visible in the filesystem
+            # even after the call to "os.unlink" returned with success. As a
+            # result, removing the directory that contains them fails and thus
+            # we get an exception.  At first we thought it could be related to
+            # the system indexer
+            # (https://superuser.com/questions/260375/why-would-system-continue-locking-
+            # executable-file-handles-after-the-app-has-exit)
+            # but this issue still occurs on systems that have it disabled.
+            #
+            # As far as we know (because we failed to pinpoint the exact reason
+            # for this condition), these issues do not reveal any bug in tests
+            # themselves, so silently ignore such errors.
+            if self.env.host.os.name == "windows":
+                self.result.log += (
+                    f"\nError while cleaning up the working directory:"
+                    f"\n{traceback.format_exc()}"
+                    f"\nHost is running Windows: discarding this error..."
+                )
+            else:
+                raise
+
     def tear_down(self) -> None:
         """Run finalization operations after a test has run.
 
@@ -316,8 +370,49 @@ class ClassicTestDriver(TestDriver):
 
         See set_up's docstring for the rationale behind this API.
         """
-        if self.working_dir_cleanup_enabled:
-            rm(self.working_dir(), True)
+        # Do nothing if cleanup is disabled, not requested for this result or
+        # disabled for this driver.
+        if (
+            self.env.cleanup_mode == CleanupMode.NONE
+            or (
+                self.env.cleanup_mode == CleanupMode.PASSING
+                and self.result.status
+                in (
+                    TestStatus.FAIL,
+                    TestStatus.XFAIL,
+                    TestStatus.XPASS,
+                    TestStatus.ERROR,
+                )
+            )
+            or not self.working_dir_cleanup_enabled
+        ):
+            return
+
+        # If an error occurs during working dir cleanup, create a dedicated
+        # error message and dump the content of the working directory tree
+        # to help investigation.
+        wd = self.working_dir()
+        try:
+            self.cleanup_working_dir()
+        except Exception:
+            result = TestResult(
+                f"{self.test_name}__tear_down",
+                env=self.test_env,
+                status=TestStatus.ERROR,
+            )
+
+            result.log += (
+                f"Error while removing the working directory {wd}:\n\n"
+            )
+            result.log += traceback.format_exc()
+            result.log += "\nRemaining files:\n"
+            for dirpath, _dirnames, filenames in os.walk(wd):
+                if dirpath != wd:
+                    result.log += f"  {os.path.relpath(dirpath, wd)}\n"
+                for f in filenames:
+                    fpath = os.path.join(dirpath, f)
+                    result.log += f"  {os.path.relpath(fpath, wd)}\n"
+            self.push_result(result)
 
     def run_wrapper(self, prev: Dict[str, Any], slot: int) -> None:
         # Make the slot (unique identifier for active jobs at a specific time)
@@ -337,7 +432,8 @@ class ClassicTestDriver(TestDriver):
             self.test_control = self.test_control_creator.create(self)
         except ValueError as exc:
             return self.push_error(
-                "Error while interpreting control: {}".format(exc))
+                "Error while interpreting control: {}".format(exc)
+            )
 
         # If test control tells us to skip the test, stop right here. Note that
         # if we have both skip and xfail, we are supposed not to execute the
@@ -352,8 +448,11 @@ class ClassicTestDriver(TestDriver):
         # If requested, prepare the test working directory to initially be a
         # copy of the test directory.
         if self.copy_test_directory:
-            sync_tree(self.test_env["test_dir"], self.test_env["working_dir"],
-                      delete=True)
+            sync_tree(
+                self.test_env["test_dir"],
+                self.test_env["working_dir"],
+                delete=True,
+            )
 
         # If the requested encoding is "binary", this actually means we will
         # handle binary data (i.e. no specific encoding). Create a binary log
@@ -377,6 +476,25 @@ class ClassicTestDriver(TestDriver):
         finally:
             self.tear_down()
 
+    def process_may_have_timed_out(self, result: ProcessResult) -> bool:
+        """
+        Return whether the process that yielded ``result`` may have timed out.
+
+        This assumes that ``result`` is the returned value from a call to the
+        ``shell`` method. Note that this uses simple heuristics to determine
+        whether the process may have timed out, as this information is not
+        preserved reliably under the hood: process is wrapped under the rlimit
+        program, which just prints a known error message and returns some
+        specific exit code in case of timeout.
+        """
+        if result.status != 2:
+            return False
+
+        if isinstance(result.out, str):
+            return bool(TIMEOUT_OUTPUT_STR_RE.search(result.out))
+        else:
+            return bool(TIMEOUT_OUTPUT_BYTES_RE.search(result.out))
+
     def compute_failures(self) -> List[str]:
         """
         Analyze the testcase result and return the list of reasons for failure.
