@@ -8,7 +8,7 @@ from typing import TYPE_CHECKING
 from e3.testsuite.result import TestStatus
 
 # Import TestsuiteCore only for typing, as this creates a circular import
-if TYPE_CHECKING:  # no cover
+if TYPE_CHECKING:
     from e3.testsuite import TestsuiteCore
 
 
@@ -30,7 +30,7 @@ def dump_xunit_report(ts: TestsuiteCore, filename: str) -> None:
     status_to_counter = {
         TestStatus.PASS: None,
         TestStatus.FAIL: "failures",
-        TestStatus.XFAIL: "failures",
+        TestStatus.XFAIL: "skipped",
         TestStatus.XPASS: None,
         TestStatus.VERIFY: None,
         TestStatus.SKIP: "skipped",
@@ -40,9 +40,11 @@ def dump_xunit_report(ts: TestsuiteCore, filename: str) -> None:
 
     # Markup to create inside <testcase> elements for each category of test
     # in XUnit.
-    counter_to_markup = {"failures": "failure",
-                         "skipped": "skipped",
-                         "errors": "error"}
+    counter_to_markup = {
+        "failures": "failure",
+        "skipped": "skipped",
+        "errors": "error",
+    }
 
     # Now create a <testcase> element for each test
     for test_name, entry in sorted(ts.report_index.entries.items()):
